////////////////////////////////////////////////////
// Blending-Programm zur Demonstration von variablen
// Prozeduren. Bei "blending" bestimmt "wahl" welcher
// Filter durchgeführt wird.
////////////////////////////////////////////////////

PImage bild1, bild2, resultat;
int w, h;
int blendingArt;

//////////////////////////////////////////////////////////
//Für blending per Addition
//////////////////////////////////////////////////////////

float additionGrau(float g1, float g2) {
  return (min(255, g1+g2));
}

color summeFarben(color farbe1, color farbe2) {

  return color( additionGrau(red(farbe1), red(farbe2)), 
    additionGrau(green(farbe1), green(farbe2)), 
    additionGrau(blue(farbe1), blue(farbe2)));
}

//////////////////////////////////////////////////////////
//Für blending per Multiplikation 
//////////////////////////////////////////////////////////

float produktGrau(float g1, float g2) {
  return (g1 * g2/255);
}

color produktFarben(color farbe1, color farbe2) {

  return color( produktGrau(red(farbe1), red(farbe2)), 
    produktGrau(green(farbe1), green(farbe2)), 
    produktGrau(blue(farbe1), blue(farbe2)));
}

//////////////////////////////////////////////////////////
//Für blending per "Bildschirm"
//////////////////////////////////////////////////////////

float bildschirmGrau(float g1, float g2) {
  float ergebnis = 255 - (255-g1) * (255-g2)/255;
  // sicherstellen, dass das Ergebnis zwischen 0 und 255 liegt:
  return constrain(ergebnis, 0, 255);
}

color bildschirmFarben(color farbe1, color farbe2) {

  return color( bildschirmGrau(red(farbe1), red(farbe2)), 
    bildschirmGrau(green(farbe1), green(farbe2)), 
    bildschirmGrau(blue(farbe1), blue(farbe2)));
}

//////////////////////////////////////////////////////////////////////
//blending wird bei jedem blend-Vorgang aufgerufen
//////////////////////////////////////////////////////////////////////

PImage blendingDurchfuehren() {

  color farbe;
  int w = bild1.width;
  int h = bild2.height;
  PImage blended = createImage(w,h,RGB);
  
  for (int i = 0; i < h; i++) {
    for (int j = 0; j < w/2; j++) {  
      color b = bild1.get(j, i);
      color s = bild2.get(j, i);  
      //hier findet die Unterscheidung nach Art des blendings statt
      switch (blendingArt) {
      case 1: 
        farbe = summeFarben( b, s); 
        break;
      case 2: 
        farbe = produktFarben( b, s); 
        break;
      case 3:
        farbe = bildschirmFarben( b, s); 
        break;
      default:
        farbe = b;
      }
      blended.set(j, i, farbe);
    }
  }
  return blended;
}


////////////////////////////////////////////////////////////////////////

void setup() {
  bild1 = loadImage("bild1.jpg");  
  bild2 = loadImage("bild2.jpg");
  w = bild1.width;  
  h = bild1.height;
  // skaliere Bild2 auf das Format von Bild1
  bild2.resize(w,h);
  
  // Fenstergröße entspricht der Größe von bild1
  surface.setSize(w, h);  

  // blending modus 
  blendingArt = 0;
  resultat = bild1;

  // Neuzeichnen des Bildschirms nur, wenn bei Benutzereingabe (redraw)
  // (und einmal am Anfang, sodass ein Bild angezeigt wird)
  noLoop();
}

//////////////////////////////////////////////////////////////////////////

void draw() {
  image(resultat,0,0);
}

//////////////////////////////////////////////////////////////////////////

void keyPressed() {

  switch (key) {
  case 'a':   //blending per Addition 
    blendingArt = 1;
    break; 

  case 'm':   //blending per Multiplikation
    blendingArt = 2;
    break;
    
  case 'b':   //blending per "Bildschirm"
    blendingArt = 3;
    break;

  case 's':   // speichere aktuelles Bild als Ergebnis ab
    save("ergebnis.jpg");
    break; 
    
  default:   // für alle anderen Eingaben zeige bild1
    blendingArt = 0;
    
  }
  
  resultat = blendingDurchfuehren();
  redraw();
}